#!/usr/bin/perl


use strict;
use utf8;
use List::Util qw(sum min max shuffle);
use Getopt::Long;
binmode STDIN, ":utf8";
binmode STDOUT, ":utf8";
binmode STDERR, ":utf8";

# This script extracts sentences from USABLE_CORPUS that cover SEED_CORPUS well.
if(@ARGV != 2) {
    print STDERR "Usage: extract-common-sentences.pl SEED_CORPUS USABLE_CORPUS\n";
    exit 1;
}

# Create n-grams from the seed corpus
open FILE, "<:utf8", $ARGV[0] or die "$ARGV[0]: $!\n";
my %seed;
while(<FILE>) {
    chomp;
    my @arr = split(/ /);
    foreach my $n (0 .. 3) {
        for(my $i = 0; $i < @arr-$n; $i++) {
            $seed{join(" ",@arr[$i .. $i+$n])}++;
        }
    }
}
close FILE;

# Order the n-grams in order of prevalence
my @ordered = sort { $seed{$b} <=> $seed{$a} } keys %seed;
for(0 .. $#ordered) {
    $seed{$ordered[$_]} = $_;
}

# Read in the sentences
open FILE, "<:utf8", $ARGV[1] or die "$ARGV[1]: $!\n";
my @sents = map { chomp $_; $_ } <FILE>;
close FILE;

# Create lists of usable sentences for each 4-gram, and a list of each 4-gram for usable sentences
my @active = map { 1 } @ordered;
my @seed2sent = map { {} } @ordered;
my @sent2seed = map { {} } @sents;
my (@score, @len);
my $has_total;
foreach my $sent (0 .. $#sents) {
    $_ = $sents[$sent];
    my @arr = split(/ /);
    $len[$sent] = @arr;
    foreach my $n (0 .. 3) {
        for(my $i = 0; $i < @arr-$n; $i++) {
            my $gram = join(" ",@arr[$i .. $i+$n]);
            if(exists $seed{$gram}) {
                if(not $seed2sent[$seed{$gram}]->{$sent}++) {
                    $sent2seed[$sent]->{$seed{$gram}}++;
                    $score[$sent]++;
                    $has_total++;
                }
            }
        }
    }
}

# Find sentences for each common n-gram in order by score
my ($covered_uniq, $covered_total);
foreach my $i (0 .. $#active) {
    # print "checking $ordered[$i]\n";
    next if not $active[$i];
    # Pick a sentence with the highest score
    my @arr = sort { $score[$b]/$len[$b] <=> $score[$a]/$len[$a] } keys %{$seed2sent[$i]};
    next if not @arr;
    print $sents[$arr[0]]."\n";
    # Deactivate all n-grams included in this sentence
    foreach my $seed (keys %{$sent2seed[$arr[0]]}) {
        if($active[$seed]) {
            foreach my $sent (keys %{$seed2sent[$seed]}) {
                $score[$sent]--;
                $covered_total++;
            }
            $covered_uniq++;
            $active[$seed] = 0;
        }
    }
    print STDERR "Chose $ordered[$i], covered $covered_uniq unique, $covered_total (".($covered_total/$has_total*100)."%) valid n-grams\n";
}
